#include <string>
#import <Foundation/Foundation.h>
#import <Cordova/CDV.h>
#import "PokktManager.h"


CDVPlugin *cordovaPlugin = nil;

@interface PokktAdDelegate:NSObject<AdDelegate>
@end


UIViewController* getRootViewController()
{
    UIViewController *viewController = [[[UIApplication sharedApplication] keyWindow] rootViewController];
    return viewController;
}

NSDictionary* extractJSON(NSString* jsonString)
{
    NSError* jsonError;
    NSData* jsonData = [jsonString dataUsingEncoding:NSUTF8StringEncoding];
    NSDictionary* jsonObject = [NSJSONSerialization JSONObjectWithData:jsonData
                                                               options:NSJSONReadingMutableContainers
                                                                 error:&jsonError];
    return jsonObject;
}

PokktConfig* parsePokktConfig(NSString* configString)
{
    NSDictionary* jsonObject = extractJSON(configString);
    
    PokktConfig* config = [[PokktConfig alloc] init];
    
    // essential
    config.securityKey = jsonObject[@"securityKey"];
    config.applicationId = jsonObject[@"applicationId"];
    config.thirdPartyUserId = jsonObject[@"thirdPartyUserId"];
    
    // optional
    config.name = jsonObject[@"name"];
    config.age = jsonObject[@"age"];
    config.sex = jsonObject[@"sex"];
    config.mobileNo = jsonObject[@"mobileNo"];
    config.emailAddress = jsonObject[@"emailAddress"];
    config.location = jsonObject[@"location"];
    config.birthday = jsonObject[@"birthday"];
    config.maritalStatus = jsonObject[@"maritalStatus"];
    config.facebookId = jsonObject[@"facebookId"];
    config.twitterHandle = jsonObject[@"twitterHandle"];
    config.education = jsonObject[@"education"];
    config.nationality = jsonObject[@"nationality"];
    config.employment = jsonObject[@"employment"];
    config.maturityRating = jsonObject[@"maturityRating"];
    
    // analytics
    config.googleTrackerID = jsonObject[@"googleAnalyticsID"];
    config.mixPanelTrackerID = jsonObject[@"mixPanelProjectToken"];
    config.flurryTrackerID = jsonObject[@"flurryApplicationKey"];
    
    if(jsonObject[@"selectedAnalyticsType"] != nil && ![jsonObject[@"selectedAnalyticsType"] isEqual:@""])
    {
        if ([jsonObject[@"selectedAnalyticsType"] isEqual:@"FLURRY"])
            config.eventType = FLURRY_ANALYTICS;
        
        else if ([jsonObject[@"selectedAnalyticsType"] isEqual:@"GOOGLE_ANALYTICS"])
            config.eventType = GOOGLE_ANALYTICS;
        
        else if ([jsonObject[@"selectedAnalyticsType"] isEqual:@"MIXPANNEL"])
            config.eventType = MIXPANNEL_ANALYTICS;
    }
    
    return config;
}

AdConfig* parseAdConfig(NSString* configString)
{
    NSDictionary* jsonObject = extractJSON(configString);
    
    AdConfig* adConfig = [[AdConfig alloc] init];
    
    adConfig.screenName = jsonObject[@"screenName"];
    adConfig.isRewarded = [jsonObject[@"isRewarded"] boolValue];
    adConfig.shouldAllowSkip = [jsonObject[@"shouldAllowSkip"] boolValue];
    adConfig.defaultSkipTime = [jsonObject[@"defaultSkipTime"] floatValue];
    adConfig.skipConfirmMessage = jsonObject[@"skipConfirmMessage"];
    adConfig.backButtonDisabled = [jsonObject[@"backButtonDisabled"] boolValue];
    adConfig.shouldAllowMute = [jsonObject[@"shouldAllowMute"] boolValue];
    adConfig.shouldConfirmSkip = [jsonObject[@"shouldSkipConfirm"] boolValue];
    adConfig.skipConfirmYesLabel = jsonObject[@"skipConfirmYesLabel"];
    adConfig.skipConfirmNoLabel = jsonObject[@"skipConfirmNoLabel"];
    adConfig.skipTimerMessage = jsonObject[@"skipTimerMessage"];
    adConfig.incentiveMessage = jsonObject[@"incentiveMessage"];
    
    return adConfig;
}

NSString* stringifyAdConfig(AdConfig* adConfig, NSDictionary* extraValues)
{
    NSMutableDictionary* jsonObject = [[NSMutableDictionary alloc] init];
    
    jsonObject[@"screenName"] = adConfig.screenName;
    jsonObject[@"isRewarded"] = adConfig.isRewarded ? @"true" : @"false";
    jsonObject[@"shouldAllowSkip"] = adConfig.shouldAllowSkip ? @"true" : @"false";
    jsonObject[@"defaultSkipTime"] = [[NSNumber numberWithInt:adConfig.defaultSkipTime] stringValue];
    jsonObject[@"skipConfirmMessage"] = adConfig.skipConfirmMessage;
    jsonObject[@"backButtonDisabled"] = adConfig.backButtonDisabled ? @"true" : @"false";
    jsonObject[@"shouldAllowMute"] = adConfig.shouldAllowMute ? @"true" : @"false";
    jsonObject[@"shouldSkipConfirm"] = adConfig.shouldConfirmSkip ? @"true" : @"false";
    jsonObject[@"skipConfirmYesLabel"] = adConfig.skipConfirmYesLabel;
    jsonObject[@"skipConfirmNoLabel"] = adConfig.skipConfirmNoLabel;
    jsonObject[@"skipTimerMessage"] = adConfig.skipTimerMessage;
    jsonObject[@"incentiveMessage"] = adConfig.incentiveMessage;
    
    if (extraValues != NULL)
        jsonObject[@"extraValues"] = extraValues;
    
    NSError* jsonError;
    NSData* jsonData = [NSJSONSerialization dataWithJSONObject:jsonObject options:0 error:&jsonError];
    NSString* jsonString = [[NSString alloc] initWithData:jsonData encoding:NSUTF8StringEncoding];
    return jsonString;
}

InAppPurchaseRequest* parseIAPDetails(NSString* detailsString)
{
    NSDictionary* jsonObject = extractJSON(detailsString);
    
    InAppPurchaseRequest* detail = [[InAppPurchaseRequest alloc] init];
    
    detail.productIdentifier = jsonObject[@"productId"];
    detail.productPrice = jsonObject[@"price"];
    detail.productCurrency = jsonObject[@"currencyCode"];
    detail.productTitle = jsonObject[@"title"];
    detail.productDescription = jsonObject[@"description"];
    
    // TODO
    //detail.setPurchaseData(jsonObject.optString("purchaseData"));
    //detail.setPurchaseSignature(jsonObject.optString("purchaseSignature"));
    //detail.setPurchaseStore(IAPStoreType.valueOf(jsonObject.optString("purchaseStore")));
    
    return detail;
}


// --------------------------- CORDOVA ------------------------------

@interface PokktNativeExtension : CDVPlugin {}

- (void) pluginInitialize;
- (void) executeSuccessCallback: (CDVInvokedUrlCommand*)command;
- (void) executeErrorCallback: (CDVInvokedUrlCommand*)command;

- (void) setDebug: (CDVInvokedUrlCommand*)command;
- (void) exportLog: (CDVInvokedUrlCommand*)command;
- (void) showLog: (CDVInvokedUrlCommand*)command;
- (void) showToast: (CDVInvokedUrlCommand*)command;
- (void) initPokkt: (CDVInvokedUrlCommand*)command;
- (void) updatePokktConfig: (CDVInvokedUrlCommand*)command;
- (void) getSDKVersion: (CDVInvokedUrlCommand*)command;

- (void) startSession: (CDVInvokedUrlCommand*)command;
- (void) endSession: (CDVInvokedUrlCommand*)command;
- (void) notifyAppInstall: (CDVInvokedUrlCommand*)command;
- (void) trackIAP: (CDVInvokedUrlCommand*)command;

- (void) getCoins: (CDVInvokedUrlCommand*)command;
- (void) getPendingCoins: (CDVInvokedUrlCommand*)command;
- (void) checkOfferWallCampaign: (CDVInvokedUrlCommand*)command;

- (void) cacheAd: (CDVInvokedUrlCommand*)command;
- (void) showAd: (CDVInvokedUrlCommand*)command;
- (void) checkAdAvailability: (CDVInvokedUrlCommand*)command;
- (void) getRewardedAdVc: (CDVInvokedUrlCommand*)command;

@end


@implementation PokktNativeExtension : CDVPlugin

- (void) pluginInitialize
{
    NSLog(@"pokkt plugin initialized!");
    
    [super pluginInitialize];
    
    // set cordova plugin
    cordovaPlugin = self;
}

- (void) executeSuccessCallback:(CDVInvokedUrlCommand*)command;
{
    CDVPluginResult *pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK];
    [self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
}

- (void) executeErrorCallback:(CDVInvokedUrlCommand*)command
{
    CDVPluginResult *pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_ERROR];
    [self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
}


/**
 * Pokkt Standard APIs
 **/

- (void) setDebug:(CDVInvokedUrlCommand*)command
{
    try
    {
        if (command.arguments.count > 0)
        {
            NSDictionary *config = [command.arguments objectAtIndex:0];
            bool isDebug = [config[@"param"] boolValue];
            [PokktManager setDebug:isDebug];
            
            [self executeSuccessCallback:command];
        }
    }
    catch (NSException *e)
    {
        NSLog(@"invalid param received! : %@", e.reason);
    }
}

- (void) exportLog:(CDVInvokedUrlCommand *)command
{
    [PokktManager exportLog:getRootViewController()];
    [self executeErrorCallback:command];
}

- (void) showLog:(CDVInvokedUrlCommand*)command
{
    NSLog(@"showLog is not supported by the iOS Plugin.");
    [self executeErrorCallback:command];
}

- (void) showToast:(CDVInvokedUrlCommand*)command
{
    NSLog(@"showToast is not supported by the iOS Plugin.");
    [self executeErrorCallback:command];
}

- (void) initPokkt:(CDVInvokedUrlCommand *)command
{
    if (command.arguments.count > 0)
    {
        NSString* paramNSStr = [command.arguments objectAtIndex:0];
        
        PokktAdDelegate *delegate = [[PokktAdDelegate alloc] init];
        [PokktManager initPokkt:parsePokktConfig(paramNSStr) withDelegate:delegate];
        
        [self executeSuccessCallback:command];
    }
    else
    {
        [self executeErrorCallback:command];
    }
}

- (void) updatePokktConfig:(CDVInvokedUrlCommand *)command
{
    NSString* paramNSStr = [command.arguments objectAtIndex:0];
    [PokktManager updatePokktConfig:parsePokktConfig(paramNSStr)];
}

- (void) getSDKVersion: (CDVInvokedUrlCommand*)command
{
    CDVPluginResult* pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK];
    NSString* result = [PokktManager getSDKVersion];
    pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK messageAsString:result];
    [self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
}


/**
 * Session related APIs
 **/

- (void) startSession:(CDVInvokedUrlCommand*)command
{
    [PokktManager startSession];
    [self executeSuccessCallback:command];
}

- (void) endSession:(CDVInvokedUrlCommand*)command
{
    [PokktManager endSession];
    [self executeSuccessCallback:command];
}

- (void) notifyAppInstall:(CDVInvokedUrlCommand *)command
{
    [PokktManager notifyAppInstall];
    [self executeSuccessCallback:command];
}

- (void) trackIAP:(CDVInvokedUrlCommand *)command
{
    NSString* paramNSStr = [command.arguments objectAtIndex:0];
    [PokktManager trackIAP:parseIAPDetails(paramNSStr)];
    [self executeSuccessCallback:command];
}


/**
 * Offerwall APIs -- NOT Supported in iOS Plugin
 **/

- (void) getCoins:(CDVInvokedUrlCommand*)command
{
    NSLog(@"getCoins is not supported by the iOS Plugin.");
    [self executeErrorCallback:command];
}

- (void) getPendingCoins:(CDVInvokedUrlCommand*)command
{
    NSLog(@"getPendignCoins is not supported by the iOS Plugin.");
    [self executeErrorCallback:command];
}

- (void) checkOfferWallCampaign:(CDVInvokedUrlCommand*)command
{
    NSLog(@"checkOfferWallCampaign is not supported by the iOS Plugin.");
    [self executeErrorCallback:command];
}


/**
 * Ad related APIs
 **/

- (void) cacheAd:(CDVInvokedUrlCommand *)command
{
    NSString* paramNSStr = [command.arguments objectAtIndex:0];
    [PokktManager cacheAd:parseAdConfig(paramNSStr)];
    [self executeSuccessCallback:command];
}

- (void) showAd:(CDVInvokedUrlCommand *)command
{
    NSString* paramNSStr = [command.arguments objectAtIndex:0];
    [PokktManager showAd:parseAdConfig(paramNSStr) presentingViewController:getRootViewController()];
    [self executeSuccessCallback:command];
}

- (void) checkAdAvailability:(CDVInvokedUrlCommand *)command
{
    NSString* paramNSStr = [command.arguments objectAtIndex:0];
    [PokktManager checkAdAvailability:parseAdConfig(paramNSStr)];
    [self executeSuccessCallback:command];
}

- (void) getRewardedAdVc:(CDVInvokedUrlCommand *)command
{
    CDVPluginResult *pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK];
    NSString* paramNSStr = [command.arguments objectAtIndex:0];
    float value = [PokktManager getRewardedAdVc:parseAdConfig(paramNSStr)];
    pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK messageAsDouble:value];
    [self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
}

@end


// ----------------------- DELEGATES ------------------------------
// events
#define POKKT_INITIALISED_EVENT         "PokktInitialised"
#define AD_CACHING_COMPLETED_EVENT      "AdCachingCompleted"
#define AD_CACHING_FAILED_EVENT         "AdCachingFailed"
#define AD_CLOSED_EVENT                 "AdClosed"
#define AD_COMPLETED_EVENT              "AdCompleted"
#define AD_DISPLAYED_EVENT              "AdDisplayed"
#define AD_SKIPPED_EVENT                "AdSkipped"
#define AD_GRATIFIED_EVENT              "AdGratified"
#define AD_AVAILABILITY_EVENT           "AdAvailability"


@implementation PokktAdDelegate

// use this to notify the framework observer
- (void) notifyFramework:(NSString*) operation param:(NSString*) param
{
    if (cordovaPlugin != nil)
    {
        NSString *event = [NSString stringWithFormat:@"javascript:cordova.fireDocumentEvent('%@', { 'param': '%@' });", operation, param];
        [[cordovaPlugin webView] stringByEvaluatingJavaScriptFromString:event];
    }
    else
    {
        NSLog(@"plugin not found!");
    }
}

- (void)onPokktInitialised:(BOOL)isReady
{
    [self notifyFramework:@POKKT_INITIALISED_EVENT param:(isReady ? @"true" : @"false")];
}

- (void)onAdCachingCompleted:(AdConfig *)adConfig withReward:(float)reward
{
    NSMutableDictionary* extraValues = [[NSMutableDictionary alloc] init];
    extraValues[@"AD_VC"] = [[NSNumber numberWithFloat:reward] stringValue];
    [self notifyFramework:@AD_CACHING_COMPLETED_EVENT param:stringifyAdConfig(adConfig, extraValues)];
}

- (void)onAdCachingFailed:(AdConfig *)adConfig withErrorMessage:(NSString *)errorMessage
{
    NSMutableDictionary* extraValues = [[NSMutableDictionary alloc] init];
    extraValues[@"MESSAGE"] = errorMessage;
    [self notifyFramework:@AD_CACHING_FAILED_EVENT param:stringifyAdConfig(adConfig, extraValues)];
}

- (void)onAdCompleted:(AdConfig *)adConfig
{
    [self notifyFramework:@AD_COMPLETED_EVENT param:stringifyAdConfig(adConfig, NULL)];
}

- (void)onAdDisplayed:(AdConfig *)adConfig
{
    [self notifyFramework:@AD_DISPLAYED_EVENT param:stringifyAdConfig(adConfig, NULL)];
}

- (void)onAdGratified:(AdConfig *)adConfig withReward:(float)reward
{
    NSMutableDictionary* extraValues = [[NSMutableDictionary alloc] init];
    extraValues[@"REWARD"] = [[NSNumber numberWithFloat:reward] stringValue];
    [self notifyFramework:@AD_GRATIFIED_EVENT param:stringifyAdConfig(adConfig, extraValues)];
}

- (void)onAdSkipped:(AdConfig *)adConfig
{
    [self notifyFramework:@AD_SKIPPED_EVENT param:stringifyAdConfig(adConfig, NULL)];
}

- (void)onAdClosed:(AdConfig *)adConfig
{
    NSMutableDictionary* extraValues = [[NSMutableDictionary alloc] init];
    extraValues[@"BACK_BUTTON"] = @"false";
    [self notifyFramework:@AD_CLOSED_EVENT param:stringifyAdConfig(adConfig, extraValues)];
}

- (void)onAdAvailabilityStatus:(AdConfig *)adConfig withAvailability:(BOOL)isAdAvailable
{
    NSMutableDictionary* extraValues = [[NSMutableDictionary alloc] init];
    extraValues[@"IS_AVAILABLE"] = (isAdAvailable ? @"true" : @"false");
    [self notifyFramework:@AD_AVAILABILITY_EVENT param:stringifyAdConfig(adConfig, extraValues)];
}

@end
