
var currentPokktConfig = null;

var PokktExtension = function () {};


/**
 * Pokkt Standard APIs
 **/

PokktExtension.prototype.setDebug = function (debug) {
    notifyNative('setDebug', undefined, debug);
};

PokktExtension.prototype.exportLog = function () {
    notifyNative('exportLog');
};

PokktExtension.prototype.showLog = function (message) {
    notifyNative('showLog', undefined, message);
};

PokktExtension.prototype.showToast = function (message) {
    notifyNative('showToast', undefined, message);
};

PokktExtension.prototype.initPokkt = function (pokktConfig) {
    currentPokktConfig = pokktConfig;
    notifyNative('initPokkt', undefined, pokktConfig);
};

PokktExtension.prototype.updatePokktConfig = function (pokktConfig) {
    currentPokktConfig = pokktConfig;
    notifyNative('updatePokktConfig', undefined, pokktConfig);
};

PokktExtension.prototype.getSDKVersion = function (callback) {
    notifyNative('getSDKVersion', callback);
};


/**
 * Session related APIs
 **/

PokktExtension.prototype.startSession = function () {
    notifyNative('startSession');
};

PokktExtension.prototype.endSession = function () {
    notifyNative('endSession');
};

PokktExtension.prototype.notifyAppInstall= function () {
    notifyNative('notifyAppInstall');
};

PokktExtension.prototype.trackIAP= function (details) {
    notifyNative('trackIAP', undefined, details);
};


/**
 * Offerwall APIs
 **/

PokktExtension.prototype.getCoins = function (assetValue) {
    currentPokktConfig.offerwallAssetValue = assetValue;
    notifyNative('getCoins', undefined, currentPokktConfig);
};

PokktExtension.prototype.getPendingCoins = function () {
    notifyNative('getPendingCoins', undefined, currentPokktConfig);
};

PokktExtension.prototype.checkOfferWallCampaign = function () {
    notifyNative('checkOfferWallCampaign', undefined, currentPokktConfig);
};


/**
 * Ad related APIs
 **/

PokktExtension.prototype.cacheAd = function(adConfig) {
    notifyNative('cacheAd', undefined, adConfig);
};

PokktExtension.prototype.showAd = function(adConfig) {
    notifyNative('showAd', undefined, adConfig);};

PokktExtension.prototype.checkAdAvailability = function(adConfig) {
    notifyNative('checkAdAvailability', undefined, adConfig);
};

PokktExtension.prototype.getRewardedAdVc = function (callback) {
    notifyNative('getRewardedAdVc', callback);
};

PokktExtension.prototype.createAdConfig = function () {
    return {
        screenName: '',
        isRewarded: false,

        defaultSkipTime: 5,
        shouldAllowSkip: true,
        shouldAllowMute: true,
        shouldConfirmSkip: true,
        skipConfirmMessage: 'Do you want to skip the video?',
        skipConfirmYesLabel: 'Okay',
        skipConfirmNoLabel: 'Nope',
        skipTimerMessage: 'Must watch for ## seconds',
        incentiveMessage: 'more seconds to go!'
    };
};

PokktExtension.prototype.createPokktConfig = function () {
    /*
         IntegrationType:
         OfferwallAndVideo - 0
         OfferwallOnly - 1
         VideoOnly - 2
     */
    return {
        // essential
        securityKey: '',
        applicationId: '',
        integrationType: 0,
        autoCaching: false,
        thirdPartyUserId: '',

        // offer wall specific
        offerwallAssetValue: 0,
        closeOnSuccessFlag: true,

        // optional
        name: '',
        age: '',
        sex: '',
        mobileNo: '',
        emailAddress: '',
        location: '',
        birthday: '',
        maritalStatus: '',
        facebookId: '',
        twitterHandle: '',
        education: '',
        nationality: '',
        employment: '',
        maturityRating: '',

        // analytics
        googleAnalyticsID: '',
        mixPanelProjectToken: '',
        flurryApplicationKey: '',
        selectedAnalyticsType: ''
    };
};


// ------- NATIVE NOTIFIERS -------

function notifyNative(operation, successCallback, param) {
    if (successCallback == null || successCallback == undefined) {
        successCallback = function (result) {
            console.log(operation + ' was successful! result: ' + result);
        };
    }

    var errorCallback = function(error) {
        console.log(operation + ' failed! error: ' + error);
    };

    var paramStr = JSON.stringify(param);
    console.log('executing operation: ' + operation + ' with param: ' + paramStr);
    cordova.exec(successCallback, errorCallback, 'PokktNativeExtension', operation, [paramStr]);
}


// ------------------------------

if (!window.plugins) {
    window.plugins = {};
}

if (!window.plugins.pokktExtension) {
    window.plugins.pokktExtension = new PokktExtension();
}

if (typeof module != 'undefined' && module.exports) {
    module.exports = PokktExtension;
}
// ------------------------------
